require 'rainbow'

namespace :dspace do
  desc "Dspace tasks"

  task :import_items, [:log] => [:environment] do |t, args|
    desc "Import DSpace items to LearningObject Repository"

    args.with_defaults(:log => STDOUT)
    logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    # Quantity of items fetched on each iteration
    limit = 1000

    # Start point from where items will be fetched
    offset = 0

    loop do

      logger.info " --> importing items from #{offset} to #{offset+limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_items_from_dspace(limit, offset, ['metadata', 'bitstreams'])
      rescue => e
        logger.warn "Database error, going to sleep"
        logger.error e
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # continue if receive an error string
        if dspace_items.is_a? String
          logger.warn "Received a string instead of items: #{dspace_items}"
          next
        end

        # Increment offset, to get new items on next iteration
        offset = offset + limit

        # Iterates through items
        # Verifies if item is already on repository
        # Initializes new LearningObjects
        # and saves them on LearningObjects's repository
        importer = Dspace::LearningObjectImporter.new(dspace_items)
        importer.import { |learning_object| logger.create "Imported: #{learning_object.id_dspace} -> #{learning_object.id}" }
      end
    end
    logger.close
  end

  private

  def dspace_repository
    DspaceService.create_client
  end

  def get_items_from_dspace(limit, offset, expand=[])
    dspace_repository.items.all(
        expand: expand.join(','),
        limit: limit, offset: offset
    )
  end

end
